=== modified file 'NEWS'
--- NEWS	2010-04-23 00:02:29 +0000
+++ NEWS	2011-02-10 17:01:10 +0000
@@ -1,8 +1,19 @@
 What's changed in loggerhead?
 =============================
 
-dev [future]
-------------
+1.19 [???]
+----------------
+
+    - If we get a HEAD request, there is no reason to expand the template, we
+      shouldn't be returning body content anyway.
+      (John Arbash Meinel, #716201, #716217)
+
+    - Merge the pqm changes back into trunk, after trunk was reverted to an old
+      revision. (John Arbash Meinel, #716152)
+
+
+1.18 [10Nov2010]
+----------------
 
     - Syntax highlighting is no longer applied for files greater than 512K,
       reducing codebrowse.launchpad.net overloading.
@@ -25,6 +36,15 @@
     - Ignore readonly+ prefix when checking if Loggerhead is serving a
       local location. (Reported by Tres Seaver.) (Matt Nordhoff)
 
+    - Set Cache-Control and Expires headers on static pages.
+      (John Arbash Meinel)
+
+    - Generate relative links where possible (everywhere but HTTP
+      redirects and feed IDs). (Michael Hudson, Matt Nordhoff)
+
+    - Fix bad redirect when visiting "/download" or "/download/".
+      (Matt Nordhoff, #247992)
+
 
 1.17 [20Aug2009]
 ----------------

=== modified file '__init__.py'
--- __init__.py	2010-04-22 08:52:59 +0000
+++ __init__.py	2011-02-10 16:19:43 +0000
@@ -30,28 +30,18 @@
 starts a web server to browse the contents of a branch.
 """
 
-version_info = (1, 17, 0)
+from info import (
+    bzr_plugin_version as version_info,
+    bzr_compatible_versions,
+    )
 
 if __name__ == 'bzrlib.plugins.loggerhead':
     import bzrlib
     from bzrlib.api import require_any_api
 
-    require_any_api(bzrlib, [
-        (1, 13, 0), (1, 15, 0), (1, 16, 0), (1, 17, 0), (1, 18, 0),
-        (2, 0, 0), (2, 1, 0), (2, 2, 0)])
-
-    # NB: Normally plugins should lazily load almost everything, but this
-    # seems reasonable to have in-line here: bzrlib.commands and options are
-    # normally loaded, and the rest of loggerhead won't be loaded until serve
-    # --http is run.
-
-    # transport_server_registry was added in bzr 1.16. When we drop support for
-    # older releases, we can remove the code to override cmd_serve.
-
-    try:
-        from bzrlib.transport import transport_server_registry
-    except ImportError:
-        transport_server_registry = None
+    require_any_api(bzrlib, bzr_compatible_versions)
+
+    from bzrlib.transport import transport_server_registry
 
     DEFAULT_HOST = '0.0.0.0'
     DEFAULT_PORT = 8080
@@ -69,10 +59,8 @@
         logging.getLogger('simpleTAL').addHandler(handler)
         logging.getLogger('simpleTALES').addHandler(handler)
 
-    def serve_http(transport, host=None, port=None, inet=None):
-        from paste.httpexceptions import HTTPExceptionHandler
-        from paste.httpserver import serve
-
+    def _ensure_loggerhead_path():
+        """Ensure that you can 'import loggerhead' and get the root."""
         # loggerhead internal code will try to 'import loggerhead', so
         # let's put it on the path if we can't find it in the existing path
         try:
@@ -81,6 +69,12 @@
             import os.path, sys
             sys.path.append(os.path.dirname(__file__))
 
+    def serve_http(transport, host=None, port=None, inet=None):
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+
+        _ensure_loggerhead_path()
+
         from loggerhead.apps.transport import BranchesFromTransportRoot
         from loggerhead.config import LoggerheadConfig
 
@@ -97,38 +91,10 @@
         app = HTTPExceptionHandler(app)
         serve(app, host=host, port=port)
 
-    if transport_server_registry is not None:
-        transport_server_registry.register('http', serve_http, help=HELP)
-    else:
-        import bzrlib.builtins
-        from bzrlib.commands import get_cmd_object, register_command
-        from bzrlib.option import Option
-
-        _original_command = get_cmd_object('serve')
-
-        class cmd_serve(bzrlib.builtins.cmd_serve):
-            __doc__ = _original_command.__doc__
-
-            takes_options = _original_command.takes_options + [
-                Option('http', help=HELP)]
-
-            def run(self, *args, **kw):
-                if 'http' in kw:
-                    from bzrlib.transport import get_transport
-                    allow_writes = kw.get('allow_writes', False)
-                    path = kw.get('directory', '.')
-                    port = kw.get('port', DEFAULT_PORT)
-                    # port might be an int already...
-                    if isinstance(port, basestring) and ':' in port:
-                        host, port = port.split(':')
-                    else:
-                        host = DEFAULT_HOST
-                    if allow_writes:
-                        transport = get_transport(path)
-                    else:
-                        transport = get_transport('readonly+' + path)
-                    serve_http(transport, host, port)
-                else:
-                    super(cmd_serve, self).run(*args, **kw)
-
-        register_command(cmd_serve)
+    transport_server_registry.register('http', serve_http, help=HELP)
+
+    def load_tests(standard_tests, module, loader):
+        _ensure_loggerhead_path()
+        standard_tests.addTests(loader.loadTestsFromModuleNames(
+            ['bzrlib.plugins.loggerhead.loggerhead.tests']))
+        return standard_tests

=== added file 'info.py'
--- info.py	1970-01-01 00:00:00 +0000
+++ info.py	2011-02-10 17:01:10 +0000
@@ -0,0 +1,14 @@
+#!/usr/bin/env python
+# API Info for loggerhead
+
+bzr_plugin_name = "loggerhead"
+
+bzr_plugin_version = (1, 18, 0)
+
+bzr_compatible_versions = [
+    (1, 17, 0), (1, 18, 0), (2, 0, 0), (2, 1, 0), (2, 2, 0), (2, 3, 0),
+    (2, 4, 0)]
+
+bzr_minimum_version = bzr_compatible_versions[0]
+
+bzr_maximum_version = bzr_compatible_versions[-1]

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2009-08-20 13:30:37 +0000
+++ loggerhead/__init__.py	2010-11-11 03:30:43 +0000
@@ -22,7 +22,7 @@
 
 import pkg_resources
 
-__version__ = '1.17'
+__version__ = '1.18'
 required_bzrlib = (1, 17)
 
 pkg_resources.get_distribution('Paste>=1.6')

=== modified file 'loggerhead/apps/__init__.py'
--- loggerhead/apps/__init__.py	2009-11-26 03:44:40 +0000
+++ loggerhead/apps/__init__.py	2010-04-30 21:30:18 +0000
@@ -9,7 +9,9 @@
 static = os.path.join(
     os.path.dirname(os.path.dirname(__file__)), 'static')
 
-static_app = urlparser.make_static(None, static)
+# Static things can be cached for half a day, we could probably make this
+# longer, except for just before rollout times.
+static_app = urlparser.make_static(None, static, cache_max_age=12*60*60)
 
 favicon_app = convert_file_errors(fileapp.FileApp(
     os.path.join(static, 'images', 'favicon.ico')))

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2010-04-14 17:54:32 +0000
+++ loggerhead/apps/branch.py	2011-02-09 22:29:57 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd.
+# Copyright (C) 2008-2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 from loggerhead.apps import static_app
 from loggerhead.controllers.annotate_ui import AnnotateUI
+from loggerhead.controllers.view_ui import ViewUI
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.diff_ui import DiffUI
@@ -89,10 +90,15 @@
             if v is not None:
                 qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
-        return request.construct_url(
-            self._environ, script_name=self._url_base,
-            path_info=unicode('/'.join(args)).encode('utf-8'),
-            querystring=qs)
+        path_info = urllib.quote(
+            unicode('/'.join(args)).encode('utf-8'), safe='/~:')
+        if qs:
+            path_info += '?' + qs
+        return self._url_base + path_info
+
+    def absolute_url(self, *args, **kw):
+        rel_url = self.url(*args, **kw)
+        return request.resolve_relative_url(rel_url, self._environ)
 
     def context_url(self, *args, **kw):
         kw = util.get_context(**kw)
@@ -119,6 +125,7 @@
         'files': InventoryUI,
         'revision': RevisionUI,
         'search': SearchUI,
+        'view': ViewUI,
         }
 
     def last_updated(self):
@@ -154,7 +161,7 @@
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
-                self._url_base + '/changes')
+                self.absolute_url('/changes'))
         if path == 'static':
             return static_app(environ, start_response)
         cls = self.controllers_dict.get(path)

=== modified file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2009-05-13 13:14:04 +0000
+++ loggerhead/apps/config.py	2010-05-05 18:35:35 +0000
@@ -208,8 +208,8 @@
                                 environ['SCRIPT_NAME']
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             response = WSGIResponse()
             self.browse(response)

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2010-03-25 16:19:24 +0000
+++ loggerhead/apps/transport.py	2010-05-05 18:35:35 +0000
@@ -33,6 +33,7 @@
 from loggerhead.apps import favicon_app, robots_app, static_app
 from loggerhead.controllers.directory_ui import DirectoryUI
 
+# TODO: Use bzrlib.ui.bool_from_string(), added in bzr 1.18
 _bools = {
     'yes': True, 'no': False,
     'on': True, 'off': False,
@@ -68,8 +69,8 @@
     def app_for_non_branch(self, environ):
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             if self.name:
                 name = self.name

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2009-10-17 08:59:33 +0000
+++ loggerhead/controllers/__init__.py	2011-02-10 17:01:10 +0000
@@ -17,8 +17,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import bzrlib.errors
 import time
 
+from paste.httpexceptions import HTTPNotFound
 from paste.request import path_info_pop, parse_querystring
 
 from loggerhead import util
@@ -97,6 +99,9 @@
         if 'Content-Type' not in headers:
             headers['Content-Type'] = 'text/html'
         writer = start_response("200 OK", headers.items())
+        if environ.get('REQUEST_METHOD') == 'HEAD':
+            # No content for a HEAD request
+            return []
         template = load_template(self.template_path)
         z = time.time()
         w = BufferingWriter(writer, 8192)
@@ -112,6 +117,10 @@
         if h is None:
             return None
         if len(self.args) > 0 and self.args != ['']:
-            return h.fix_revid(self.args[0])
+            try:
+                revid = h.fix_revid(self.args[0])
+            except bzrlib.errors.NoSuchRevision:
+                raise HTTPNotFound;
         else:
-            return h.last_revid
+            revid = h.last_revid
+        return revid

=== added file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/annotate_ui.py	2010-12-01 08:30:02 +0000
@@ -0,0 +1,54 @@
+#
+# Copyright (C) 2010 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+from loggerhead.controllers.view_ui import ViewUI
+from loggerhead import util
+
+class AnnotateUI(ViewUI):
+
+    def annotate_file(self, info):
+        file_id = info['file_id']
+        revid = info['change'].revid
+        
+        tree = self.tree_for(file_id, revid)
+        
+        change_cache = {}
+        last_line_revid = None
+        parity = 1
+        for line_revid, text in tree.annotate_iter(file_id):
+            if line_revid == last_line_revid:
+                # remember which lines have a new revno and which don't
+                new_rev = False
+            else:
+                new_rev = True
+                parity ^= 1
+                last_line_revid = line_revid
+                if line_revid in change_cache:
+                    change = change_cache[line_revid]
+                else:
+                    change = self._history.get_changes([line_revid])[0]
+                    change_cache[line_revid] = change
+
+            yield util.Container(
+                parity=parity, new_rev=new_rev, change=change)
+            
+    def get_values(self, path, kwargs, headers):
+        values = super(AnnotateUI, self).get_values(path, kwargs, headers)
+        values['annotated'] = self.annotate_file(values)
+        
+        return values

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2010-04-08 01:08:33 +0000
+++ loggerhead/controllers/download_ui.py	2010-05-05 19:03:40 +0000
@@ -44,8 +44,8 @@
             args.append(arg)
 
         if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url(
-                      '../changes'))
+            raise httpexceptions.HTTPMovedPermanently(
+                self._branch.absolute_url('/changes'))
 
         revid = h.fix_revid(args[0])
         file_id = args[1]

=== renamed file 'loggerhead/controllers/annotate_ui.py' => 'loggerhead/controllers/view_ui.py'
--- loggerhead/controllers/annotate_ui.py	2009-10-17 06:35:33 +0000
+++ loggerhead/controllers/view_ui.py	2010-12-01 08:30:02 +0000
@@ -35,20 +35,18 @@
 from loggerhead import util
 
 
-class AnnotateUI(TemplatedBranchView):
-
-    template_path = 'loggerhead.templates.annotate'
-
-    def annotate_file(self, file_id, revid):
-        z = time.time()
-        lineno = 1
-        parity = 0
-
+class ViewUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.view'
+    
+    def tree_for(self, file_id, revid):
         file_revid = self._history.get_inventory(revid)[file_id].revision
-        tree = self._history._branch.repository.revision_tree(file_revid)
+        return self._history._branch.repository.revision_tree(file_revid)
 
+    def text_lines(self, file_id, revid):
         file_name = os.path.basename(self._history.get_path(revid, file_id))
-
+        
+        tree = self.tree_for(file_id, revid)
         file_text = tree.get_file_text(file_id)
         encoding = 'utf-8'
         try:
@@ -58,44 +56,27 @@
             file_text = file_text.decode(encoding)
 
         file_lines = bzrlib.osutils.split_lines(file_text)
+        # This can throw bzrlib.errors.BinaryFile (which our caller catches).
+        bzrlib.textfile.check_text_lines(file_lines)
+        
+        if highlight is not None:
+            hl_lines = highlight(file_name, file_text, encoding)
+            # highlight strips off extra newlines at the end of the file.
+            extra_lines = len(file_lines) - len(hl_lines)
+            hl_lines.extend([u''] * extra_lines)
+        else:
+            hl_lines = map(cgi.escape, file_lines)
+        
+        return hl_lines;
 
+    def file_contents(self, file_id, revid):
         try:
-            bzrlib.textfile.check_text_lines(file_lines)
+            file_lines = self.text_lines(file_id, revid)
         except bzrlib.errors.BinaryFile:
             # bail out; this isn't displayable text
-            yield util.Container(parity=0, lineno=1, status='same',
-                                 text='(This is a binary file.)',
-                                 change=util.Container())
-        else:
-            if highlight is not None:
-                hl_lines = highlight(file_name, file_text, encoding)
-                hl_lines.extend([u''] * (len(file_lines) - len(hl_lines)))
-            else:
-                hl_lines = map(cgi.escape, file_lines)
-
-            change_cache = {}
-
-            last_line_revid = None
-            for line_revid, text in tree.annotate_iter(file_id):
-                if line_revid == last_line_revid:
-                    # remember which lines have a new revno and which don't
-                    status = 'same'
-                else:
-                    status = 'changed'
-                    parity ^= 1
-                    last_line_revid = line_revid
-                    if line_revid in change_cache:
-                        change = change_cache[line_revid]
-                    else:
-                        change = self._history.get_changes([line_revid])[0]
-                        change_cache[line_revid] = change
-
-                yield util.Container(
-                    parity=parity, lineno=lineno, status=status,
-                    change=change, text=hl_lines[lineno - 1])
-                lineno += 1
-
-        self.log.debug('annotate: %r secs' % (time.time() - z,))
+            return ['(This is a binary file.)']
+
+        return file_lines
 
     def get_values(self, path, kwargs, headers):
         history = self._history
@@ -105,7 +86,7 @@
         file_id = kwargs.get('file_id', None)
         if (file_id is None) and (path is None):
             raise HTTPBadRequest('No file_id or filename '
-                                 'provided to annotate')
+                                 'provided to view')
 
         if file_id is None:
             file_id = history.get_file_id(revid, path)
@@ -140,13 +121,17 @@
         branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
 
         return {
+            # In AnnotateUI, "annotated" is a generator giving revision
+            # numbers per lines, but the template checks if "annotated" is
+            # true or not before using it, so we have to define it here also.
+            'annotated': False,
             'revno_url': revno_url,
             'file_id': file_id,
-            'path': path,
+            'file_path': path,
             'filename': filename,
             'navigation': navigation,
             'change': change,
-            'contents': list(self.annotate_file(file_id, revid)),
+            'contents':  self.file_contents(file_id, revid),
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,

=== modified file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	2010-04-22 03:23:55 +0000
+++ loggerhead/highlight.py	2010-12-01 08:30:02 +0000
@@ -16,6 +16,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import bzrlib.osutils
 import cgi
 
 from pygments import highlight as _highlight_func
@@ -36,7 +37,7 @@
     """
 
     if len(text) > MAX_HIGHLIGHT_SIZE:
-        return map(cgi.escape, text.split('\n'))
+        return map(cgi.escape,  bzrlib.osutils.split_lines(text))
 
     formatter = HtmlFormatter(style=style, nowrap=True, classprefix='pyg-')
 
@@ -48,6 +49,7 @@
         except (ClassNotFound, ValueError):
             lexer = TextLexer(encoding=encoding)
 
-    hl_lines = _highlight_func(text, lexer, formatter).split('\n')
+    hl_lines = _highlight_func(text, lexer, formatter)
+    hl_lines = bzrlib.osutils.split_lines(hl_lines)
 
     return hl_lines

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2010-04-26 21:03:47 +0000
+++ loggerhead/history.py	2010-05-10 13:48:25 +0000
@@ -152,6 +152,9 @@
                 filename=rich_filename(paths[1], kind),
                 file_id=file_id))
 
+# The lru_cache is not thread-safe, so we need a lock around it for
+# all threads.
+rev_info_memory_cache_lock = threading.RLock()
 
 class RevInfoMemoryCache(object):
     """A store that validates values against the revids they were stored with.
@@ -175,7 +178,11 @@
         If a value was stored under `key`, with the same revid, return it.
         Otherwise return None.
         """
-        cached = self._cache.get(key)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            cached = self._cache.get(key)
+        finally:
+            rev_info_memory_cache_lock.release()
         if cached is None:
             return None
         stored_revid, data = cached
@@ -187,7 +194,11 @@
     def set(self, key, revid, data):
         """Store `data` under `key`, to be checked against `revid` on get().
         """
-        self._cache[key] = (revid, data)
+        rev_info_memory_cache_lock.acquire()
+        try:
+            self._cache[key] = (revid, data)
+        finally:
+            rev_info_memory_cache_lock.release()
 
 # Used to store locks that prevent multiple threads from building a 
 # revision graph for the same branch at the same time, because that can

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2009-10-17 06:55:25 +0000
+++ loggerhead/static/css/global.css	2010-04-23 01:30:29 +0000
@@ -96,27 +96,27 @@
 	text-align:center;
 	color:#fff;
 	text-decoration:none;
-	background:#2f40a5;
+	background:#d18b39;
 }
 ul#submenuTabs li {
 	float:left;
 	display:inline;
 	margin-left:1px;
 	color:#fff;
-	background:#2f40a5;
+	background:#d18b39;
 }
 ul#submenuTabs li a:hover {
 	float:left;
 	display:inline;
 	color:#fff;
-	background:#2f40a5;
+	background:#d18b39;
 	text-decoration:underline;
 }
 ul#submenuTabs li#first a, ul#submenuTabs li#first a:link, ul#submenuTabs li#first a:visited, ul#submenuTabs li#first a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) top left no-repeat;
+	background:#d18b39 url(../images/bg_submenuTabs.gif) top left no-repeat;
 }
 ul#submenuTabs li#last a, ul#submenuTabs li#last a:link, ul#submenuTabs li#last a:visited, ul#submenuTabs li#last a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) bottom right no-repeat;
+	background:#d18b39 url(../images/bg_submenuTabs.gif) bottom right no-repeat;
 }
 
 /* Search & RSS */

=== renamed file 'loggerhead/static/css/annotate.css' => 'loggerhead/static/css/view.css'
--- loggerhead/static/css/annotate.css	2009-04-30 10:39:05 +0000
+++ loggerhead/static/css/view.css	2010-12-01 08:30:02 +0000
@@ -1,29 +1,31 @@
 /*table*/
-.annoLineTit, .annoLine, .annoRevTit, .annoRev, .annoComm, .annoCommTit, .annoContTit, .annoCont {
-	width:45px;
+.viewLineTit, .viewLine, .viewRevTit, .viewRev,
+.viewContTit, .viewCont
+{
 	border:1px solid #d2d2d2;
 }
-.annoLine, .annoRev, .annoComm, .annoCont {
+.viewLine, .viewRev, .viewComm, .viewCont {
 	border:none;
 }
-.annoLine {
+.viewLine, .viewLineTit {
+	padding-right: .2em;
+}
+.viewLine {
 	width:37px;
 }
-.annoContTit, .annoCont {
+.viewContTit, .viewCont {
 	width:auto;
+	padding-left: .3em;
 }
-.annoRevTit, .annoRev {
+.viewRevTit, .viewRev {
 	width:70px;
 	text-align:center;
 }
-.annoComm, .annoCommTit {
-	width:200px;
-}
-.annoLine, .annoCont {
+.viewLine, .viewCont {
 	font:normal 12px/normal monospace;
         whitespace: pre;
 }
-.annoCont pre { margin: 0; }
-.annoLine {
+.viewCont pre { margin: 0; }
+.viewLine {
 	text-align:right;
 }

=== modified file 'loggerhead/static/images/bg_Tabs.gif'
Binary files loggerhead/static/images/bg_Tabs.gif and loggerhead/static/images/bg_Tabs.gif differ
=== modified file 'loggerhead/static/images/bg_menuTabs.gif'
Binary files loggerhead/static/images/bg_menuTabs.gif and loggerhead/static/images/bg_menuTabs.gif differ
=== modified file 'loggerhead/static/images/bg_submenuTabs.gif'
Binary files loggerhead/static/images/bg_submenuTabs.gif and loggerhead/static/images/bg_submenuTabs.gif differ
=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-10-17 08:47:38 +0000
+++ loggerhead/templatefunctions.py	2010-12-01 08:30:02 +0000
@@ -121,9 +121,9 @@
 
 
 @templatefunc
-def annotate_link(url, revno, path):
+def view_link(url, revno, path):
     return '<a href="%s" title="Annotate %s">%s</a>' % (
-        url(['/annotate', revno, path]), cgi.escape(path), cgi.escape(path))
+        url(['/view', revno, path]), cgi.escape(path), cgi.escape(path))
 
 @templatefunc
 def revision_link(url, revno, path, frag=''):

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2009-06-06 12:23:11 +0000
+++ loggerhead/templates/atom.pt	2010-05-05 18:42:58 +0000
@@ -4,8 +4,8 @@
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
   <updated tal:content="string:${updated}Z">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.url(['/atom'])" />
+  <id tal:content="python:branch.absolute_url(['/atom'])">url</id>
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.absolute_url(['/atom'])" />
   <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
@@ -16,7 +16,7 @@
       updated
     </updated>
     <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <id tal:content="python:branch.absolute_url(['/revision', entry.revno])">
       ID
     </id>
     <author tal:repeat="author entry/authors">

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2009-10-17 06:55:25 +0000
+++ loggerhead/templates/inventory.pt	2010-12-01 08:30:02 +0000
@@ -89,11 +89,11 @@
 
             <!-- Show this if it's a symlink -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='symlink'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', change.revno, file.absolutepath])">
+                <td class="autcell"><a tal:attributes="href python:url(['/view', change.revno, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file_flecha.gif')" alt="Symlink" />
                     </a>
 
-                 <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])"
+                 <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                         tal:content="file/filename" class="link"></a>
                  </td>
                  <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
@@ -113,17 +113,17 @@
 
             <!-- Show this if it's a regular file -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='file'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])">
+                <td class="autcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif');
-                                             title string:Annotate ${file/filename}"
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is False" />
                         <!-- Show a different icon id the file executable -->
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file_modify.gif');
-                                             title string:Annotate ${file/filename}"
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is True" alt="File" />
                     </a>
 
-              <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])"
+              <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                      tal:content="file/filename" class="link"></a></td>
               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                   title string:Show revision ${file/change/revno}"
@@ -131,8 +131,8 @@
               </td>
               <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
               <td class="timedate2" tal:content="python:util.human_size(file.size)"></td>
-              <td class="expcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath]);
-                                                     title string:Annotate ${file/filename}">
+              <td class="expcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath]);
+                                                     title string:View ${file/filename}">
                                       <img tal:attributes="src python:branch.static_url('/static/images/ico_planilla.gif')" alt="Diff" />
                                      </a>
               </td>

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2009-06-24 02:35:46 +0000
+++ loggerhead/templates/macros.pt	2009-07-21 04:39:33 +0000
@@ -43,14 +43,6 @@
 
 <body>
     <!-- Loggerhead Content Area -->
-      <div id="finderBox">
-         <tal:search-box define="navigation navigation"
-                        replace="structure python:search_box(branch,
-                                 navigation)" />
-        <div>
-          <tal:feed-link replace="structure python:feed_link(branch, url)" />
-        </div>
-      </div>
 
       <tal:menu define="fileview_active fileview_active"
                 replace="structure python:menu(branch, url, fileview_active)" />

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-04-28 03:01:54 +0000
+++ loggerhead/templates/revision.pt	2010-12-01 08:30:02 +0000
@@ -20,7 +20,6 @@
     </head>
 
     <body>
-
       <tal:block metal:fill-slot="heading">
         <h1>
           <tal:has-link condition="branch/branch_link">
@@ -42,7 +41,7 @@
             </tal:compare-to>
           </span>
           <span class="breadcrumb" tal:condition="specific_path">
-            : <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+            : <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
           </span>
         </h1>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -55,7 +54,7 @@
           </a>
         </p>
         <p tal:condition="specific_path">
-          Viewing changes to <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+          Viewing changes to <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
         </p>
         <ul id="submenuTabs">
           <li id="first"><a tal:attributes="href python:url(['/files', change.revno]);

=== renamed file 'loggerhead/templates/annotate.pt' => 'loggerhead/templates/view.pt'
--- loggerhead/templates/annotate.pt	2009-04-30 10:39:05 +0000
+++ loggerhead/templates/view.pt	2010-12-01 08:30:02 +0000
@@ -3,12 +3,12 @@
 <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
   <head>
     <title metal:fill-slot="title"
-           tal:content="string:${branch/friendly_name} : contents of ${path}
+           tal:content="string:${branch/friendly_name} : contents of ${file_path}
                         at revision ${change/revno}">
     </title>
     <metal:block fill-slot="header_extras">
         <link rel="stylesheet" type="text/css" media="all"
-              tal:attributes="href python:branch.static_url('/static/css/annotate.css')"/>
+              tal:attributes="href python:branch.static_url('/static/css/view.css')"/>
         <link rel="stylesheet" type="text/css" media="all"
               tal:attributes="href python:branch.static_url('/static/css/highlight.css')"/>
     </metal:block>
@@ -36,6 +36,14 @@
             <li id="first">
                 <a tal:attributes="href python:url(['/files', change.revno], clear=1)">browse files</a>
             </li>
+            <li tal:condition="not:annotated">
+              <a tal:attributes="href python:url(['/annotate', revno_url, file_path], clear=1)"
+              >view revision numbers per line</a>
+            </li>
+            <li tal:condition="annotated">
+              <a tal:attributes="href python:url(['/view', revno_url, file_path], clear=1)"
+              >view without revision numbers</a>
+            </li>
             <li>
                 <a tal:attributes="href python:url(['/revision', change.revno], clear=1)">view revision</a>
             </li>
@@ -50,25 +58,30 @@
             </li>
         </ul>
 
-      <div class="annotate">
+      <div class="view">
         <table id="logentries">
           <tr class="logheader">
-            <td class="annoLineTit">Line</td>
-            <td class="annoRevTit">Revision</td>
-            <td class="annoContTit">Contents</td>
+            <td class="viewLineTit">Line</td>
+            <td class="viewRevTit" tal:condition="annotated">Revision</td>
+            <td class="viewContTit">Contents</td>
           </tr>
 
-          <tr tal:repeat="line contents"
-              tal:attributes="class string:blueRow${line/parity}">
-            <td class="annoLine"><a tal:attributes="id string:L${line/lineno}; href string:#L${line/lineno}" tal:content="line/lineno">1</a></td>
-            <td class="annoRev">
-              <a tal:condition="python:line.status=='changed'"
-                 tal:content="python:util.trunc(line.change.revno)"
-                 tal:attributes="href python:url(['/revision', line.change.revno], clear=1);
-                                 title python:'%s by %s, on %s (%s)'%(line.change.revno, ', '.join(util.hide_emails(line.change.authors)), line.change.date.strftime('%d %b %Y %H:%M'), util.date_time(line.change.date))"></a>
-            </td>
-            <td class="annoCont"><pre tal:content="structure line/text"></pre></td>
-          </tr>
+          <tal:rep tal:repeat="line contents">
+            <tr tal:define="anno python:annotated and annotated.next()"
+                tal:attributes="class python:anno and 'blueRow' + str(anno.parity) or None">
+              <td class="viewLine">
+                <a tal:attributes="id string:L${repeat/line/number}; href string:#L${repeat/line/number}"
+                   tal:content="repeat/line/number">1</a>
+              </td>
+              <td class="viewRev" tal:condition="annotated">
+                <a tal:condition="python:anno.new_rev"
+                   tal:content="python:util.trunc(anno.change.revno)"
+                   tal:attributes="href python:url(['/revision', anno.change.revno], clear=1);
+                                   title python:'%s by %s, on %s (%s)' % (anno.change.revno, ', '.join(util.hide_emails(anno.change.authors)), anno.change.date.strftime('%d %b %Y %H:%M'), util.date_time(anno.change.date))"></a>
+              </td>
+              <td class="viewCont"><pre tal:content="structure line"></pre></td>
+            </tr>
+          </tal:rep>
         </table>
       </div>
 

=== modified file 'loggerhead/tests/__init__.py'
--- loggerhead/tests/__init__.py	2006-12-11 06:44:19 +0000
+++ loggerhead/tests/__init__.py	2010-05-10 19:36:37 +0000
@@ -0,0 +1,26 @@
+# Copyright 2006, 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+def load_tests(standard_tests, module, loader):
+    standard_tests.addTests(loader.loadTestsFromModuleNames([
+        (__name__ + '.' + x) for x in [
+            'test_controllers',
+            'test_corners',
+            'test_simple',
+            'test_templating',
+        ]]))
+    return standard_tests

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2009-03-19 18:48:15 +0000
+++ loggerhead/tests/test_controllers.py	2011-02-10 17:01:10 +0000
@@ -1,3 +1,10 @@
+from cStringIO import StringIO
+import logging
+
+from paste.httpexceptions import HTTPServerError
+
+from bzrlib import errors
+
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
@@ -15,15 +22,50 @@
         tree.commit('')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, '')
+        branch_app.log.setLevel(logging.CRITICAL)
+        # These are usually set in BranchWSGIApp.app(), which is set from env
+        # settings set by BranchesFromTransportRoot, so we fake it.
+        branch_app._static_url_base = '/'
+        branch_app._url_base = '/'
         return tree.branch, InventoryUI(branch_app, branch_app.get_history)
 
+    def consume_app(self, app, extra_environ=None):
+        env = {'SCRIPT_NAME': '/files', 'PATH_INFO': ''}
+        if extra_environ is not None:
+            env.update(extra_environ)
+        body = StringIO()
+        start = []
+        def start_response(status, headers, exc_info=None):
+            start.append((status, headers, exc_info))
+            return body.write
+        extra_content = list(app(env, start_response))
+        body.writelines(extra_content)
+        return start[0], body.getvalue()
+
     def test_get_filelist(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
         inv = bzrbranch.repository.get_inventory(bzrbranch.last_revision())
         self.assertEqual(1, len(inv_ui.get_filelist(inv, '', 'filename')))
 
+    def test_smoke(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui)
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertContainsRe(content, 'filename')
+
+    def test_no_content_for_HEAD(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui,
+                            extra_environ={'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertEqual('', content)
+
 
 class TestRevisionUI(BasicTests):
 
@@ -61,20 +103,25 @@
 
     def make_annotate_ui_for_file_history(self, file_id, rev_ids_texts):
         tree = self.make_branch_and_tree('.')
-        open('filename', 'w').write('')
+        self.build_tree_contents([('filename', '')])
         tree.add(['filename'], [file_id])
         for rev_id, text in rev_ids_texts:
-            open('filename', 'w').write(text)
+            self.build_tree_contents([('filename', text)])
             tree.commit(rev_id=rev_id, message='.')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, friendly_name='test_name')
         return AnnotateUI(branch_app, branch_app.get_history)
 
     def test_annotate_file(self):
         history = [('rev1', 'old\nold\n'), ('rev2', 'new\nold\n')]
         ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
-        annotated = list(ann_ui.annotate_file('file_id', 'rev2'))
+        # A lot of this state is set up by __call__, but we'll do it directly
+        # here.
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = list(annotate_info['annotated'])
         self.assertEqual(2, len(annotated))
         self.assertEqual('2', annotated[0].change.revno)
         self.assertEqual('1', annotated[1].change.revno)

=== modified file 'loggerhead/trace.py'
--- loggerhead/trace.py	2008-10-24 02:26:05 +0000
+++ loggerhead/trace.py	2010-11-12 00:09:49 +0000
@@ -22,6 +22,7 @@
 
 import os
 import logging
+import logging.handlers
 import sys
 
 

